from llama_index.readers.gitlab.issues.base import GitLabIssuesReader
from llama_index.readers.gitlab.repository.base import GitLabRepositoryReader


__all__ = [
    "GitLabIssuesReader",
    "GitLabRepositoryReader",
]
